"""
Tests data reading and writing operation, along with condition generation
"""
import pytest
import numpy as np
import pandas as pd
import os
from shutil import rmtree
from numpy.testing import assert_equal, assert_allclose
from xsugar import Experiment
from ast import literal_eval

def testSavePSDFigureFilename(exp, path_data):
    """
    Tests that we successfully created and saved a single PSD figure when
    our dataset is just a single item.
    """
    raw_data = pd.DataFrame({'Time (ms)': [1, 2, 3],
                             'Current (mV)': [4,4.5,6]})
    cond = {'wavelength': 1, 'temperature': 25, 'frequency': 8500}
    condition_name = 'TEST1~wavelengths-1~temperatures-25'
    filename_desired = condition_name + '~PSD.png'
    exp.data = {condition_name: raw_data}
    exp.plotPSD(average_along=None)
    file_found = os.path.isfile(path_data['figures_full_path'] + filename_desired)
    assert_equal(file_found, True)

def testSavePSDFigureMultipleFilename(exp, path_data):
    """
    Tests that we successfully created and saved a single PSD figure when
    our dataset is just a single item.
    """
    raw_data = pd.DataFrame({'Time (ms)': [1, 2, 3],
                             'Current (mV)': [4,4.5,6]})
    cond = {'wavelength': 1, 'temperature': 25, 'frequency': 8500}
    condition_name_1 = 'TEST1~wavelengths-1~temperatures-25~replicate-0'
    condition_name_2 = 'TEST1~wavelengths-1~temperatures-25~replicate-1'
    filename_desired_1 = condition_name_1 + '~PSD.png'
    filename_desired_2 = condition_name_2 + '~PSD.png'
    exp.data = {condition_name_1: raw_data, condition_name_2: raw_data}
    exp.plotPSD(average_along=None)
    file1_found = os.path.isfile(path_data['figures_full_path'] +
                                 filename_desired_1)
    file2_found = os.path.isfile(path_data['figures_full_path'] +
                                 filename_desired_2)
    assert_equal(file1_found, True)
    assert_equal(file2_found, True)

def testSavePSDFigureAverageFilename(exp, path_data):
    """
    Tests that we successfully created and saved a single PSD figure when
    we want to create an averaged PSD plot
    """
    raw_data = pd.DataFrame({'Time (ms)': [1, 2, 3],
                             'Current (mV)': [4,4.5,6]})
    raw_data_2 = pd.DataFrame({'Time (ms)': [1, 2, 3],
                             'Current (mV)': [8,4.5,8]})
    cond = {'wavelength': 1, 'temperature': 25, 'frequency': 8500}
    condition_name_1 = 'TEST1~wavelengths-1~temperatures-25~replicate-1'
    condition_name_2 = 'TEST1~wavelengths-1~temperatures-25~replicate-2'
    filename_desired = 'TEST1~wavelengths-1~temperatures-25~PSD~averaged.png'

    exp.data = {condition_name_1: raw_data,
                 condition_name_2: raw_data_2}
    exp.plotPSD(average_along='replicate')
    file_found = os.path.isfile(path_data['figures_full_path'] + filename_desired)
    assert_equal(file_found, True)

def testGenerateTimeDomainPlot(exp, path_data):
    """
    Tests that we successfully create a simple figure from a single pandas
    array.
    """
    raw_data = pd.DataFrame({'Time (ms)': [1, 2, 3],
                             'Current (mV)': [4,4.5,6]})
    cond = {'wavelength': 1, 'temperature': 25, 'frequency': 8500}
    condition_name = 'TEST1~wavelengths-1~temperatures-25_replicate-1'
    filename_desired = condition_name + '.png'
    exp.data = {condition_name: raw_data}
    exp.plot()
    file_found = os.path.isfile(path_data['figures_full_path'] + filename_desired)
    assert_equal(file_found, True)

def testGenerateRepresentativePlot(exp, path_data):
    """
    Tests that we successfully create a single figure from a whole set of
    replicate data, instead of a bunch of figures
    """
    raw_data = pd.DataFrame({'Time (ms)': [1, 2, 3],
                             'Current (mV)': [4,4.5,6]})
    cond_1 = {'wavelength': 1, 'temperature': 25, 'frequency': 8500,
              'replicate': 1}
    cond_2 = {'wavelength': 1, 'temperature': 25, 'frequency': 8500,
              'replicate': 2}
    condition_name_1 = 'TEST1~wavelengths-1~temperatures-25~replicate-1'
    condition_name_2 = 'TEST1~wavelengths-1~temperatures-25~replicate-2'

    filename_desired = 'TEST1~wavelengths-1~temperatures-25~representative'
    exp.data = {condition_name_1: raw_data,
                 condition_name_2 : raw_data}
    exp.plot(representative='replicate')
    files_found = os.listdir(path_data['figures_full_path'])
    file_1_found = \
        os.path.isfile(path_data['figures_full_path'] + filename_desired + '.png')
    assert_equal(file_1_found, True)
    assert_equal(len(files_found), 1)



"""
Adds the source files to the path for files in any subdirectory
"""
import os
import sys

file_location = os.path.dirname(os.path.abspath(__file__))
base_location= os.path.abspath(file_location)

sys.path.insert(0, base_location)



import pytest
import numpy as np
import pandas as pd
from pandas.testing import assert_frame_equal
import os
from shutil import rmtree
from numpy.testing import assert_equal, assert_allclose
from xsugar import Experiment
from ast import literal_eval
from itertools import zip_longest
from spectralpy import power_spectrum
from xsugar import assertDataDictEqual

@pytest.fixture
def exp(path_data):
    wavelengths = np.array([1, 2, 3])
    temperatures = np.array([25, 50])
    frequency = 8500
    exp = Experiment(name='TEST1', kind='test',
                     frequency=frequency,
                     wavelengths=wavelengths,
                     temperatures=temperatures)
    yield exp
    rmtree(path_data['data_base_path'], ignore_errors=True)
    rmtree(path_data['figures_base_path'], ignore_errors=True)
    rmtree(path_data['designs_base_path'], ignore_errors=True)


def testGenerateMasterData1Var(exp):
    scalar_data = {'TEST1~wavelength-1~temperature-25': 1,
                   'TEST1~wavelength-1~temperature-35': 2}
    desired_data = pd.DataFrame({
        'temperature': [25, 35],
        'Value': [1, 2]})
    actual_data = exp.master_data(data_dict=scalar_data)
    assert_frame_equal(actual_data, desired_data)

def testGenerateMasterData2Var(exp):
    scalar_data = {'TEST1~wavelength-1~temperature-25': 1,
                   'TEST1~wavelength-1~temperature-35': 2,
                   'TEST1~wavelength-2~temperature-25': 3,
                   'TEST1~wavelength-2~temperature-35': 4}
    desired_data = pd.DataFrame({
        'wavelength': [1, 1, 2, 2],
        'temperature': [25, 35, 25, 35],
        'Value': [1, 2, 3, 4]})
    actual_data = exp.master_data(data_dict=scalar_data)
    assert_frame_equal(actual_data, desired_data)

def test_data_from_master(exp):
    master_data = pd.DataFrame({
        'wavelength': [1, 2],
        'Value': [3, 4]})
    desired_data = {
        'TEST1~wavelength-1': 3,
        'TEST1~wavelength-2': 4,
    }
    actual_data = exp.data_from_master(master_data)
    assertDataDictEqual(actual_data, desired_data)

def test_data_from_master_2var(exp):
    master_data = pd.DataFrame({
        'wavelength': [0, 1, 2, 0, 1, 2],
        'temperature': [25.0, 25.0, 25.0, 35.0, 35.0, 35.0],
        'Value': [0, 1, 2, 3, 4, 5]})
    desired_data_dict = {
        'TEST1~wavelength-0~temperature-25.0': 0,
        'TEST1~wavelength-1~temperature-25.0': 1,
        'TEST1~wavelength-2~temperature-25.0': 2,
        'TEST1~wavelength-0~temperature-35.0': 3,
        'TEST1~wavelength-1~temperature-35.0': 4,
        'TEST1~wavelength-2~temperature-35.0': 5}
    actual_data_dict = exp.data_from_master(master_data)
    assertDataDictEqual(actual_data_dict, desired_data_dict)

def testGenerateMasterDataDict1Var(exp):
    data_dict = {'TEST1~wavelength-1': 3.0,
        'TEST1~wavelength-2': 4.0}
    desired_data = {
        'TEST1~wavelength-all': pd.DataFrame({
                'wavelength': [1, 2],
                'Value': [3.0, 4.0]})}
    actual_data = exp.master_data_dict(data_dict)
    assertDataDictEqual(actual_data, desired_data)

def testGenerateMasterDataDict2Var(exp):
    master_data = pd.DataFrame({
        'wavelength': [0, 1, 2, 0, 1, 2],
        'temperature': [25.0, 25.0, 25.0, 35.0, 35.0, 35.0],
        'Value': [0, 1, 2, 3, 4, 5]})
    data_dict = {
        'TEST1~wavelength-0~temperature-25.0': 0,
        'TEST1~wavelength-1~temperature-25.0': 1,
        'TEST1~wavelength-2~temperature-25.0': 2,
        'TEST1~wavelength-0~temperature-35.0': 3,
        'TEST1~wavelength-1~temperature-35.0': 4,
        'TEST1~wavelength-2~temperature-35.0': 5}
    desired_data = {
        'TEST1~wavelength-all':
        {
            'TEST1~temperature-25.0': pd.DataFrame({
                'wavelength': [0, 1, 2],
                'Value': [0, 1, 2]}),
            'TEST1~temperature-35.0': pd.DataFrame({
                'wavelength': [0,1,2],
                'Value': [3, 4, 5]})
        },
        'TEST1~temperature-all':
        {
            'TEST1~wavelength-0': pd.DataFrame({
                'temperature': [25.0, 35.0],
                'Value': [0, 3]}),
            'TEST1~wavelength-1': pd.DataFrame({
                'temperature': [25.0, 35.0],
                'Value': [1, 4] }),
            'TEST1~wavelength-2': pd.DataFrame({
                'temperature': [25.0, 35.0],
                'Value': [2, 5] })
        },
    }

    actual_data = exp.master_data_dict(data_dict)
    assertDataDictEqual(actual_data, desired_data)

def testGenerateMasterDataDict3Var(exp):
    master_data = pd.DataFrame({
        'wavelength': [0, 0, 0, 0, 1, 1, 1, 1],
        'temperature': [25.0, 25.0, 35.0, 35.0, 25.0, 25.0, 35.0, 35.0],
        'material': ['Au', 'Al', 'Au', 'Al', 'Au', 'Al', 'Au', 'Al'],
        'Value': [0, 1, 2, 3, 4, 5, 6, 7]})
    data_dict = {
        'TEST1~wavelength-0~temperature-25.0~material-Au': 0,
        'TEST1~wavelength-0~temperature-25.0~material-Al': 1,
        'TEST1~wavelength-0~temperature-35.0~material-Au': 2,
        'TEST1~wavelength-0~temperature-35.0~material-Al': 3,
        'TEST1~wavelength-1~temperature-25.0~material-Au': 4,
        'TEST1~wavelength-1~temperature-25.0~material-Al': 5,
        'TEST1~wavelength-1~temperature-35.0~material-Au': 6,
        'TEST1~wavelength-1~temperature-35.0~material-Al': 7}
    desired_data = {
        'TEST1~wavelength-all~material-Au':
        {
            'TEST1~temperature-25.0': pd.DataFrame({
                'wavelength': [0, 1],
                'Value': [0, 4]}),
            'TEST1~temperature-35.0': pd.DataFrame({
                'wavelength': [0, 1],
                'Value': [2, 6]}),
        },
        'TEST1~wavelength-all~material-Al':
        {
            'TEST1~temperature-25.0': pd.DataFrame({
                'wavelength': [0,1],
                'Value': [1,5]}),
            'TEST1~temperature-35.0': pd.DataFrame({
                'wavelength': [0,1],
                'Value': [3,7]}),
        },
        'TEST1~wavelength-all~temperature-25.0': {
            'TEST1~material-Au': pd.DataFrame({
                'wavelength': [0,1],
                'Value': [0,4]}),
            'TEST1~material-Al': pd.DataFrame({
                'wavelength': [0,1],
                'Value': [1,5]}),
        },
        'TEST1~wavelength-all~temperature-35.0': {
            'TEST1~material-Au': pd.DataFrame({
                'wavelength': [0,1],
                'Value': [2,6]}),
            'TEST1~material-Al': pd.DataFrame({
                'wavelength': [0,1],
                'Value': [3,7]}),
        },
        'TEST1~temperature-all~material-Au': {
            'TEST1~wavelength-0': pd.DataFrame({
                'temperature': [25.0,35.0],
                'Value': [0,2]}),
            'TEST1~wavelength-1': pd.DataFrame({
                'temperature': [25.0,35.0],
                'Value': [4,6]}),
        },
        'TEST1~temperature-all~material-Al': {
            'TEST1~wavelength-0': pd.DataFrame({
                'temperature': [25.0,35.0],
                'Value': [1,3]}),
            'TEST1~wavelength-1': pd.DataFrame({
                'temperature': [25.0,35.0],
                'Value': [5,7]}),
        },
        'TEST1~temperature-all~wavelength-0': {
            'TEST1~material-Au': pd.DataFrame({
                'temperature': [25.0,35.0],
                'Value': [0,2]}),
            'TEST1~material-Al': pd.DataFrame({
                'temperature': [25.0,35.0],
                'Value': [1,3]}),
        },
        'TEST1~temperature-all~wavelength-1': {
            'TEST1~material-Au': pd.DataFrame({
                'temperature': [25.0,35.0],
                'Value': [4,6]}),
            'TEST1~material-Al': pd.DataFrame({
                'temperature': [25.0,35.0],
                'Value': [5,7]}),
        },
        'TEST1~material-all~temperature-25.0': {
            'TEST1~wavelength-0': pd.DataFrame({
                'material': ['Au', 'Al'],
                'Value': [0,1]}),
            'TEST1~wavelength-1': pd.DataFrame({
                'material': ['Au', 'Al'],
                'Value': [4,5]}),
        },
        'TEST1~material-all~temperature-35.0': {
            'TEST1~wavelength-0': pd.DataFrame({
                'material': ['Au', 'Al'],
                'Value': [2,3]}),
            'TEST1~wavelength-1': pd.DataFrame({
                'material': ['Au', 'Al'],
                'Value': [6,7]}),
        },
        'TEST1~material-all~wavelength-0': {
            'TEST1~temperature-25.0': pd.DataFrame({
                'material': ['Au', 'Al'],
                'Value': [0,1]}),
            'TEST1~temperature-35.0': pd.DataFrame({
                'material': ['Au', 'Al'],
                'Value': [2,3]}),
        },
        'TEST1~material-all~wavelength-1': {
            'TEST1~temperature-25.0': pd.DataFrame({
                'material': ['Au', 'Al'],
                'Value': [4,5]}),
            'TEST1~temperature-35.0': pd.DataFrame({
                'material': ['Au', 'Al'],
                'Value': [6,7]}),
        },
    }

    actual_data = exp.master_data_dict(data_dict)
    assertDataDictEqual(actual_data, desired_data)



from numpy.testing import assert_equal
from pandas.testing import assert_frame_equal
from itertools import zip_longest
from pandas import DataFrame

def assertDataDictEqual(data_dict_actual, data_dict_desired):
    assert_equal(type(data_dict_actual), dict)
    assert_equal(type(data_dict_desired), dict)
    for actual_name, desired_name in \
        zip_longest(data_dict_actual.keys(),
                    data_dict_desired.keys()):
        assert_equal(actual_name, desired_name)

    for actual_data, desired_data in \
         zip_longest(data_dict_actual.values(),
                     data_dict_desired.values()):
        if isinstance(actual_data, DataFrame):
            assert_frame_equal(actual_data, desired_data)
        elif isinstance(actual_data, dict):
            assertDataDictEqual(actual_data, desired_data)
        else:
            assert_equal(actual_data, desired_data)



import pytest
import numpy as np
from shutil import rmtree
from xsugar import Experiment
from pathlib import Path
import os

@pytest.fixture
def exp(exp_data):
    exp = Experiment(name='TEST1', kind='test',
                     frequency=exp_data['frequency'],
                     wavelength=exp_data['wavelength'],
                     temperature=exp_data['temperature'])
    yield exp
    rmtree(exp_data['data_base_path'], ignore_errors=True)
    rmtree(exp_data['figures_base_path'], ignore_errors=True)
    rmtree(exp_data['designs_base_path'], ignore_errors=True)

@pytest.fixture
def path_data():
    base_path = str(Path.home())
    if 'LOGNAME' in os.environ:
        if os.environ['LOGNAME'] == 'jordan.e':
            base_path = '/Users/jordan.e/Google Drive/GTD/PhD/docs/' +\
                'pockels_modulator'
    data_base_path = base_path + '/data/test/'
    data_full_path = base_path + '/data/test/TEST1/'
    figures_base_path = base_path + '/figures/test/'
    figures_full_path = base_path + '/figures/test/TEST1/'
    designs_base_path = base_path + '/designs/TEST1/'
    yield {
        'data_base_path': data_base_path,
        'data_full_path': data_full_path,
        'figures_base_path': figures_base_path,
        'figures_full_path': figures_full_path,
        'designs_base_path': designs_base_path,
    }

@pytest.fixture
def exp_data(path_data):
    wavelength = np.array([1, 2, 3])
    temperature = np.array([25, 50])
    frequency = 8500
    yield dict({
        'wavelength': wavelength,
        'temperature': temperature,
        'frequency': frequency,
    }, **path_data)



"""
Tests data reading and writing operation, along with condition generation
"""
import pytest
import numpy as np
import pandas as pd
from pandas.testing import assert_frame_equal
import os
from shutil import rmtree
from numpy.testing import assert_equal, assert_allclose
from xsugar import Experiment, assertDataDictEqual
from ast import literal_eval
from itertools import zip_longest
from pathlib import Path


def testExtractConstants(exp, exp_data):
    """
    Tests that our experiment properly extracts metadata from a list of
    arbitrary keyword arguments.
    """
    actual_constants = exp.constants
    desired_constants = {'frequency': exp_data['frequency']}
    assert_equal(actual_constants , desired_constants)


def test_extract_factors(exp, exp_data):
    """
    Tests proper extraction of conditions from arbitrary input keyword
    arguments.
    """
    actual_factors = exp.factors
    desired_factors = {
        'wavelength': exp_data['wavelength'],
        'temperature': exp_data['temperature']}
    assert_equal(actual_factors, desired_factors)

def test_generate_conditions(exp, exp_data):
    """
    Tests that our generator actually generates all the right combinations
    and in the right order.
    """
    expected_conds = [
        {'wavelength': 1, 'temperature': 25, 'frequency': 8500},
        {'wavelength': 1, 'temperature': 50, 'frequency': 8500},
        {'wavelength': 2, 'temperature': 25, 'frequency': 8500},
        {'wavelength': 2, 'temperature': 50, 'frequency': 8500},
        {'wavelength': 3, 'temperature': 25, 'frequency': 8500},
        {'wavelength': 3, 'temperature': 50, 'frequency': 8500},
    ]
    for actual_cond, desired_cond in  zip(exp.conditions,
                                          expected_conds):
        assert_equal(actual_cond, desired_cond)

def testSaveRawResultsFilename(exp, exp_data):
    """
    Tests that we save the raw results in the proper directory and with
    the proper metadata and with the proper name.
    """
    raw_data = pd.DataFrame({'wavelengths': [1, 2, 3],
                             'Current': [4,4.5,6]})
    cond = {'wavelengths': 1, 'temperatures': 25, 'frequency': 8500}
    exp.saveRawResults(raw_data, cond)
    filename_desired = 'TEST1~wavelengths-1~temperatures-25.csv'
    file_found = os.path.isfile(exp_data['data_full_path'] + filename_desired)
    assert_equal(file_found, True)

def testSaveRawResultsMetadata(exp, exp_data):
    """
    Tests that we save the raw results in the proper directory and with
    the proper metadata and with the proper name.
    """
    raw_data = pd.DataFrame({'wavelengths': [1, 2, 3],
                             'Current': [4,4.5,6]})
    cond = {'wavelengths': 1, 'temperatures': 25, 'frequency':
            exp_data['frequency']}
    exp.saveRawResults(raw_data, cond)
    filename_desired = 'TEST1~wavelengths-1~temperatures-25.csv'
    with open(exp_data['data_full_path'] + filename_desired) as fh:
        first_line = fh.readline()
        metadata_actual = literal_eval(first_line)

    metadata_desired = {'frequency': exp_data['frequency']}
    assert_equal(metadata_actual, metadata_desired)

def testSaveRawResultsData(exp, exp_data):
    """
    Tests that we correctly save the raw data and can read it out again.
    """
    data_desired = pd.DataFrame({'wavelengths': [1, 2, 3],
                             'Current': [4,4.5,6]})
    cond = {'wavelengths': 1, 'temperatures': 25, 'frequency':
            exp_data['frequency']}
    exp.saveRawResults(data_desired, cond)
    filename_desired = 'TEST1~wavelengths-1~temperatures-25.csv'
    with open(exp_data['data_full_path'] + filename_desired) as fh:
        first_line = fh.readline()
        data_actual = pd.read_csv(fh)
    assert_allclose(data_actual, data_desired)
    assert_equal(data_actual.columns.values, data_desired.columns.values)

@pytest.mark.skip
def testSaveDerivedQuantitiesFilename(exp, exp_data):
    master_data = {'TEST1': pd.DataFrame({'wavelengths': [1, 2, 3],
                             'Mean': [1,2,4]})}
    cond = {'wavelengths': 1, 'temperatures': 25, 'frequency': 8500}
    exp.master_data = master_data
    exp.saveDerivedQuantities()
    filename_desired = 'TEST1.csv'
    file_found = os.path.isfile(exp_data['data_full_path'] + filename_desired)
    assert_equal(file_found, True)

@pytest.mark.skip
def testSaveDerivedQuantitiesData(exp, exp_data):
    data_desired = pd.DataFrame({'wavelengths': [1, 2, 3],
                             'Mean': [1,2,4]})
    exp.master_data = data_desired
    exp.saveDerivedQuantities()
    filename_desired = 'TEST1.csv'
    with open(exp_data['data_full_path'] + filename_desired) as fh:
        metadata = fh.readline()
        data_actual = pd.read_csv(fh)
    assert_allclose(data_actual, data_desired)
    assert_equal(data_actual.columns.values, data_desired.columns.values)

@pytest.mark.skip
def testSaveDerivedQuantitiesMetadata(exp, exp_data):
    data_desired = pd.DataFrame({'wavelengths': [1, 2, 3],
                             'Mean': [1,2,4]})
    metadata_desired = {'frequency': exp_data['frequency']}
    exp.master_data = data_desired
    exp.saveDerivedQuantities()
    filename_desired = 'TEST1.csv'
    with open(exp_data['data_full_path'] + filename_desired) as fh:
        metadata_actual = literal_eval(fh.readline())
    assert_equal(metadata_actual, metadata_desired)

def testLoadData(exp, exp_data):
    data_desired = pd.DataFrame({'wavelengths': [1, 2, 3],
                             'Mean': [1,2,4]})
    metadata_desired = {'non': 'sense'}
    filename = 'TEST1~wavelengths-1~temperatures-25'
    file_extension = '.csv'
    full_filename = exp_data['data_full_path'] + filename + file_extension
    with open(full_filename, 'w+') as fh:
        fh.write(str(metadata_desired) + '\n')
        data_desired.to_csv(fh, mode='a', index=False)
    exp.loadData()
    data_actual = exp.data[filename]
    assert_frame_equal(data_actual, data_desired)

def testLoadXRDData():
    file_location = os.path.dirname(os.path.abspath(__file__))
    base_path = file_location + '/data'
    exp = Experiment(name='TEST1', kind='test', base_path=base_path)
    exp.loadXRDData()
    data_desired = pd.DataFrame({
        'Angle (deg)': [69.05, 69.055, 69.06, 69.065, 69.07,69.075,69.08,
        69.085, 69.09, 69.095, 69.1, 69.105, 69.11, 69.115],
        'Counts': [24, 30, 28, 40, 132, 272, 3472, 16368,21970,10562,
                   1210,264,130,64]})
    data_actual = exp.data['TEST1~1~type-locked_coupled~peak-Si']
    assert_frame_equal(data_actual, data_desired)

def testLoadXRDMetadata():
    file_location = os.path.dirname(os.path.abspath(__file__))
    base_path = file_location + '/data'
    exp = Experiment(name='TEST1', kind='test', base_path=base_path)
    exp.loadXRDData()
    metadata_desired = {
        'date': '02/10/2021',
        'increment': 0.005, 'scantype': 'locked coupled',
        'start': 69.05, 'steps': 14, 'time': 1,
        'theta': 34.0, '2theta': 68.0, 'phi': 180.13, 'chi': -0.972}
    metadata_actual = exp.metadata['TEST1~1~type-locked_coupled~peak-Si']
    assert_equal(metadata_actual, metadata_desired)

def testLoadConstants(exp, exp_data):
    """
    Tests that we can load metadata from a file successfully
    """
    wavelengths = np.array([1, 2, 3])
    temperatures = np.array([25, 50])
    frequency = 8500
    with open(exp_data['data_full_path'] + \
              'TEST1~wavelengths-1~temperatures-25.csv', 'w+') as fh:
        fh.write('{"frequency": 8500}\n')
        fh.write(f'Time, Data\n')
        fh.write(f'1, 2\n')
    exp = Experiment(name='TEST1', kind='test')
    exp.loadData()
    constants_actual = exp.constants
    constants_desired = {'frequency': frequency}
    assert_equal(constants_actual, constants_desired)

def testLoadMetadata(exp, exp_data):
    """
    Tests that we can load metadata from a file successfully
    """
    wavelengths = np.array([1, 2, 3])
    temperatures = np.array([25, 50])
    frequency = 8500
    with open(exp_data['data_full_path'] + \
              'TEST1~wavelengths-1~temperatures-25.csv', 'w+') as fh:
        fh.write('{"frequency": 8500}\n')
        fh.write(f'Time, Data\n')
        fh.write(f'1, 2\n')
    exp = Experiment(name='TEST1', kind='test')
    exp.loadData()
    metadata_actual = exp.metadata
    metadata_desired = {'TEST1~wavelengths-1~temperatures-25': \
                        {'frequency': frequency}}
    assert_equal(metadata_actual, metadata_desired)

# TODO: ADD TEST CASE TO ENSURE WE DON'T LOAD IN TOO MUCH DATA, OR DATA
# THAT DOES NOT PRECISELY MATCH *BOTH* THE NAME *AND* THE ID.

# TODO: ADD TEST CASE TO ENSURE WE DON'T LOAD IN TOO MUCH DATA, OR DATA
# THAT DOES NOT PRECISELY MATCH *BOTH* THE NAME *AND* THE ID.

def testLookup(exp):
    fudge_data = pd.DataFrame(
        {'Time (ms)': [1, 2, 3],
        'Voltage (V)': [0,0.1, 0.2]})
    exp.data = {'TEST1~wavelengths-1~temperatures-25':fudge_data,
                     'TEST1~wavelengths-2~temperatures-25':fudge_data,
                     'TEST1~wavelengths-2~temperatures-35':fudge_data,
                     'TEST1~wavelengths-2~temperatures-35':fudge_data,
                    }
    data_actual = exp.lookup(temperatures=25)
    data_desired = {'TEST1~wavelengths-1~temperatures-25':fudge_data,
                     'TEST1~wavelengths-2~temperatures-25':fudge_data,}
    assertDataDictEqual(data_actual, data_desired)



"""
Tests data reading and writing operation, along with condition generation
"""
import pytest
import numpy as np
import pandas as pd
from pandas.testing import assert_frame_equal
import os
from shutil import rmtree
from numpy.testing import assert_equal, assert_allclose
from xsugar import Experiment
from ast import literal_eval
from itertools import zip_longest
from spectralpy import power_spectrum
from xsugar import assertDataDictEqual

@pytest.fixture
def exp(path_data):
    wavelengths = np.array([1, 2, 3])
    temperatures = np.array([25, 50])
    frequency = 8500
    exp = Experiment(name='TEST1', kind='test',
                     frequency=frequency,
                     wavelengths=wavelengths,
                     temperatures=temperatures)
    yield exp
    rmtree(path_data['data_base_path'], ignore_errors=True)
    rmtree(path_data['figures_base_path'], ignore_errors=True)
    rmtree(path_data['designs_base_path'], ignore_errors=True)

def test_average_data_scalar(exp):
    fudge_data_1 = 2.0
    fudge_data_2 = 3.0
    averaged_data = 2.5
    condition_1 = {'frequency': 8500,
                   'wavelength': 1,
                   'replicate': 0}
    condition_2 = {'frequency': 8500,
                   'wavelength': 1,
                   'replicate': 1}
    name_1 = 'TEST1~wavelength-1~replicate-0'
    name_2 = 'TEST1~wavelength-1~replicate-1'
    data_dict = {name_1: fudge_data_1, name_2: fudge_data_2}
    group_name = 'TEST1~wavelength-1'

    averaged_data_desired = {group_name: averaged_data}
    averaged_data_actual = exp.average_data(data_dict, average_along='replicate')
    assertDataDictEqual(averaged_data_actual, averaged_data_desired)

def test_average_data_pandas(exp):
    fudge_data_1 = pd.DataFrame({'Time (ms)': [1, 2, 3],
                               'Photocurrent (nA)': [0.5, 0.6, 0.7]})
    fudge_data_2 = pd.DataFrame({'Time (ms)': [1, 2, 3],
                               'Photocurrent (nA)': [1, 1.2, 1.4]})
    averaged_data = pd.DataFrame({'Time (ms)': [1, 2, 3],
                               'Photocurrent (nA)': [0.75, 0.9, 1.05]})
    condition_1 = {'frequency': 8500,
                   'wavelength': 1,
                   'replicate': 0}
    condition_2 = {'frequency': 8500,
                   'wavelength': 1,
                   'replicate': 1}
    name_1 = 'TEST1~wavelength-1~replicate-0'
    name_2 = 'TEST1~wavelength-1~replicate-1'
    group_name = 'TEST1~wavelength-1'
    data_dict = {name_1: fudge_data_1, name_2: fudge_data_2}

    averaged_data_desired = {group_name: averaged_data}
    averaged_data_actual_first = exp.average_data(data_dict,
            average_along='replicate', averaging_type='first')
    averaged_data_actual_last = exp.average_data(data_dict,
            average_along='replicate', averaging_type='first')
    assertDataDictEqual(averaged_data_actual_first, averaged_data_desired)
    assertDataDictEqual(averaged_data_actual_last, averaged_data_desired)

def testExtractDerivedQuantityMean(exp):
    """
    Attempst to extract the mean from a set of data
    """
    fudge_data_1 = pd.DataFrame({'Time (ms)': [1, 2, 3],
                               'Photocurrent (nA)': [0.5, 0.6, 0.7]})
    fudge_data_2 = pd.DataFrame({'Time (ms)': [1, 2, 3],
                               'Photocurrent (nA)': [1, 1.2, 1.4]})
    mean_data_1 = 0.6
    mean_data_2 = 1.2
    name_1 = 'TEST1~wavelength-1~replicate-0'
    name_2 = 'TEST1~wavelength-1~replicate-1'
    desired_quantities = {name_1: mean_data_1, name_2: mean_data_2}
    data_dict = {name_1: fudge_data_1, name_2: fudge_data_2}

    def getPhotocurrentMean(pandas_dict, cond):
        return np.mean(pandas_dict['Photocurrent (nA)'].values)

    actual_quantities = exp.derived_quantity(
        data_dict=data_dict, quantity_func=getPhotocurrentMean,
        average_along=None)

    assertDataDictEqual(actual_quantities, desired_quantities)

def testExtractDerivedQuantityPSD(exp):
    """
    Attempts to extract the PSD from a set of data
    """
    fudge_data_1 = pd.DataFrame({'Time (ms)': [1, 2, 3],
                               'Photocurrent (nA)': [0.5, 0.6, 0.7]})
    fudge_data_2 = pd.DataFrame({'Time (ms)': [1, 2, 3],
                               'Photocurrent (nA)': [1, 1.2, 1.4]})
    name_1 = 'TEST1~wavelength-1~replicate-0'
    name_2 = 'TEST1~wavelength-1~replicate-1'
    data_dict = {name_1: fudge_data_1, name_2: fudge_data_2}

    def powerSpectrum(data, cond):
        return power_spectrum(data)
    actual_psd_data = exp.derived_quantity(
        data_dict=data_dict, quantity_func=powerSpectrum)
    desired_psd_data = {name_1: power_spectrum(fudge_data_1),
                        name_2: power_spectrum(fudge_data_2)}
    assertDataDictEqual(actual_psd_data, desired_psd_data)

def testExtractDerivedQuantityPSDAverage(exp):
    fudge_data_1 = pd.DataFrame({'Time (ms)': [1, 2, 3],
                               'Photocurrent (nA)': [0.5, 0.6, 0.7]})
    fudge_data_2 = pd.DataFrame({'Time (ms)': [1, 2, 3],
                               'Photocurrent (nA)': [1, 1.2, 1.4]})
    name_1 = 'TEST1~wavelength-1~replicate-0'
    name_2 = 'TEST1~wavelength-1~replicate-1'
    data_dict = {name_1: fudge_data_1, name_2: fudge_data_2}
    data_psd_1 = power_spectrum(fudge_data_1)
    data_psd_2 = power_spectrum(fudge_data_2)
    data_psd_1.iloc[:, 1:] += data_psd_2.iloc[:,1:]
    data_psd_1.iloc[:, 1:] /= 2

    def powerSpectrum(data, cond):
        return power_spectrum(data)

    desired_psd = data_psd_1
    desired_data_dict = {'TEST1~wavelength-1': desired_psd}
    actual_data_dict = exp.derived_quantity(
        data_dict=data_dict, quantity_func=powerSpectrum,
        average_along='replicate')
    assertDataDictEqual(actual_data_dict, desired_data_dict)



"""
Tests data reading and writing operation, along with condition generation
"""
import pytest
import numpy as np
import pandas as pd
import os
from shutil import rmtree
from numpy.testing import assert_equal, assert_allclose
from xsugar import Experiment
from ast import literal_eval

def testNameFromCondition(exp):
    """
    Tests whether we can properly generate a name from a specified
    condition
    """
    filename_desired = 'TEST1~wavelength-1~temperature-25'
    condition = {'wavelength': 1, 'temperature': 25}
    filename_actual = exp.nameFromCondition(condition)
    assert_equal(filename_actual, filename_desired)

def testNameFromConditionWithID(exp):
    """
    Tests whether we can properly generate a name from a specified
    condition
    """
    filename_desired = 'TEST1-id~wavelength-1~temperature-25'
    condition = {'ident': 'id', 'wavelength': 1, 'temperature': 25}
    filename_actual = exp.nameFromCondition(condition)
    assert_equal(filename_actual, filename_desired)

def testConditionFromName(exp):
    """
    Tests whether we can generate a condition from a specified name
    """
    filename_desired = 'TEST1~wavelength-1~temperature-25'
    condition_desired = {'wavelength': 1, 'temperature': 25,
                         'frequency': 8500}
    condition_actual = exp.conditionFromName(filename_desired)
    assert_equal(condition_actual, condition_desired)

def testConditionFromNamePartial(exp):
    """
    Tests whether we can generate a condition from a specified name
    """
    filename_desired = 'TEST1~wavelength-1~temperature-25'
    condition_desired = {'wavelength': 1, 'temperature': 25}
    condition_actual = exp.conditionFromName(
        filename_desired, full_condition=False)
    assert_equal(condition_actual, condition_desired)

def testConditionFromNameMetadata(exp):
    filename_desired = 'TEST1~wavelength-1~temperature-25'
    condition_desired = {'wavelength': 1, 'temperature': 25,
                         'frequency': 8500,
                         'horn': 'shoe'}
    exp.metadata['TEST1~wavelength-1~temperature-25'] = \
        {'horn': 'shoe'}

    condition_actual = exp.conditionFromName(filename_desired)
    assert_equal(condition_actual, condition_desired)


def testConditionFromNameWithID(exp):
    """
    Tests whether we can generate a condition from a specified name
    """
    filename_desired = 'TEST1-id~wavelength-1~temperature-25'
    condition_desired = {'ident': 'id', 'wavelength': 1,
                         'temperature': 25,
                         'frequency': 8500}
    condition_actual = exp.conditionFromName(filename_desired)
    assert_equal(condition_actual, condition_desired)




import pytest
import numpy as np
import pandas as pd
import os
from shutil import rmtree
from numpy.testing import assert_equal, assert_allclose
from xsugar import Experiment

def test_get_conditions(exp):
    exp.data = {
        'TEST1~wavelength-1~temperature-25': None,
        'TEST1~wavelength-2~temperature-25': None,
        'TEST1~wavelength-3~temperature-25': None,
        'TEST1~wavelength-1~temperature-35': None,
        'TEST1~wavelength-2~temperature-35': None,
        'TEST1~wavelength-3~temperature-35': None,}
    desired_conditions = [
        {'wavelength': 1, 'temperature': 25},
        {'wavelength': 2, 'temperature': 25},
        {'wavelength': 3, 'temperature': 25},
        {'wavelength': 1, 'temperature': 35},
        {'wavelength': 2, 'temperature': 35},
        {'wavelength': 3, 'temperature': 35},
    ]
    actual_conditions = exp.get_conditions()
    assert_equal(actual_conditions, desired_conditions)

def test_get_conditions_exclude(exp):
    exp.data = {
        'TEST1~wavelength-1~temperature-25': None,
        'TEST1~wavelength-2~temperature-25': None,
        'TEST1~wavelength-3~temperature-25': None,
        'TEST1~wavelength-1~temperature-35': None,
        'TEST1~wavelength-2~temperature-35': None,
        'TEST1~wavelength-3~temperature-35': None,}
    desired_conditions = [
        {'wavelength': 1},
        {'wavelength': 2},
        {'wavelength': 3},
    ]
    actual_conditions = exp.get_conditions(exclude='temperature')
    assert_equal(actual_conditions, desired_conditions)

def test_get_conditions_exclude_uneven(exp):
    exp.data = {
        'TEST1~wavelength-1~temperature-25': None,
        'TEST1~wavelength-2~temperature-25': None,
        'TEST1~wavelength-3~temperature-25': None,
        'TEST1~wavelength-1~temperature-35': None,
        'TEST1~wavelength-2~temperature-35': None,
        'TEST1~wavelength-3~temperature-35': None,
        'TEST1~wavelength-4~temperature-35': None,}
    desired_conditions = [
        {'wavelength': 1},
        {'wavelength': 2},
        {'wavelength': 3},
        {'wavelength': 4},
    ]
    actual_conditions = exp.get_conditions(exclude='temperature')
    assert_equal(actual_conditions, desired_conditions)

def test_factors_from_condition(exp):
    actual_factors = exp.factors_from_condition(
            {'wavelength': 2, 'temperature': 5, 'material': 'Al'})
    desired_factors = ['wavelength', 'temperature', 'material']
    assert_equal(actual_factors, desired_factors)

def test_append_condition(exp):
    desired_conditions = [
        {'wavelength': 1, 'temperature': 25, 'frequency': 8500},
        {'wavelength': 1, 'temperature': 50, 'frequency': 8500},
        {'wavelength': 2, 'temperature': 25, 'frequency': 8500},
        {'wavelength': 2, 'temperature': 50, 'frequency': 8500},
        {'wavelength': 3, 'temperature': 25, 'frequency': 8500},
        {'wavelength': 3, 'temperature': 50, 'frequency': 8500},
        {'wavelength': 5, 'temperature': 0, 'glue': 9, 'frequency': 8500}
    ]
    exp.append_condition(wavelength=5, temperature=0, glue=9)
    actual_conditions = exp.conditions
    assert_equal(actual_conditions, desired_conditions)

def test_insert_condition(exp):
    desired_conditions = [
        {'wavelength': 1, 'temperature': 25, 'frequency': 8500},
        {'wavelength': 1, 'temperature': 50, 'frequency': 8500},
        {'wavelength': 5, 'temperature': 0, 'glue': 9, 'frequency': 8500},
        {'wavelength': 2, 'temperature': 25, 'frequency': 8500},
        {'wavelength': 2, 'temperature': 50, 'frequency': 8500},
        {'wavelength': 3, 'temperature': 25, 'frequency': 8500},
        {'wavelength': 3, 'temperature': 50, 'frequency': 8500},
    ]
    exp.insert_condition(2, wavelength=5, temperature=0, glue=9)
    actual_conditions = exp.conditions
    assert_equal(actual_conditions, desired_conditions)

def test_generate_conditions_1to1(exp):
    desired_conditions = [
        {'wavelength': 1, 'temperature': 25, 'frequency': 8500},
        {'wavelength': 2, 'temperature': 50, 'frequency': 8500},
        {'wavelength': 3, 'temperature': 75, 'frequency': 8500},
    ]
    actual_conditions = exp.generate_conditions(
            comb_type='1to1',
            wavelength=[1, 2, 3], temperature=[25,50,75])
    assert_equal(actual_conditions, desired_conditions)




import pytest 
import numpy as np
import os
from shutil import rmtree
from numpy.testing import assert_equal, assert_allclose
from xsugar import Experiment

def testDirectoryAbsence(exp_data):
    """
    Tests that our test directories are properly created and destroyed
    """
    data_exists = os.path.isdir(exp_data['data_base_path'])
    figures_exist = os.path.isdir(exp_data['figures_base_path'])
    design_exist = os.path.isdir(exp_data['designs_base_path'])
    assert_equal(data_exists, False)
    assert_equal(figures_exist, False)
    assert_equal(design_exist, False)

def testDirectoryCreation(exp, exp_data):
    """
    Tests that we properly create the right directories and subdirectories
    """
    data_exists = os.path.isdir(exp_data['data_base_path'])
    figures_exist = os.path.isdir(exp_data['figures_base_path'])
    assert_equal(data_exists, True)
    assert_equal(figures_exist, True)

    data_exists = os.path.isdir(exp_data['data_base_path'] + 'TEST1/')
    figures_exist = os.path.isdir(exp_data['figures_base_path'] + 'TEST1/')
    assert_equal(data_exists, True)
    assert_equal(figures_exist, True)

def testDesignDirectoryCreation(exp, exp_data):
    """
    Tests that we properly create the right directories and subdirectories
    """
    exp = Experiment(name='TEST1', kind='design')
    data_exists = os.path.isdir(exp_data['designs_base_path'] + 'data/')
    figures_exist = os.path.isdir(exp_data['designs_base_path'] + 'figures/')
    assert_equal(data_exists, True)
    assert_equal(figures_exist, True)


def testNormalPath(exp, exp_data):
    """
    Tests that non-design paths resolve to the right locations with the
    format base/kind/name/.
    """
    desired_figures_path = exp_data['figures_full_path']
    desired_data_path = exp_data['data_full_path']
    actual_figures_path = exp.figures_full_path
    actual_data_path = exp.data_full_path
    assert_equal(actual_data_path, desired_data_path)
    assert_equal(actual_figures_path, desired_figures_path)

def testDesignPath(exp_data):
    exp = Experiment(name='TEST1', kind='design')
    actual_figures_path = exp.figures_full_path
    actual_data_path = exp.data_full_path
    desired_figures_path = exp_data['designs_base_path'] + 'figures/'
    desired_data_path = exp_data['designs_base_path'] + 'data/'

    assert_equal(actual_data_path, desired_data_path)
    assert_equal(actual_figures_path, desired_figures_path)



import pytest
from xsugar import Experiment, assertDataDictEqual
from pandas.testing import assert_frame_equal
from numpy.testing import assert_equal
import numpy as np
import pandas as pd
from itertools import zip_longest
from shutil import rmtree

@pytest.fixture
def exp(path_data):
    wavelengths = np.array([1, 2, 3])
    temperatures = np.array([25, 50])
    frequency = 8500
    fudge_data_1 = pd.DataFrame({'Time (ms)': [1, 2, 3],
                               'Photocurrent (nA)': [0.5, 1.3, 0.7]})
    fudge_data_2 = pd.DataFrame({'Time (ms)': [1, 2, 3],
                               'Photocurrent (nA)': [0.5, 0.6, 0.7]})
    condition_1 = {'frequency': frequency,
                   'wavelength': 1,
                   'replicate': 0}
    condition_2 = {'frequency': frequency,
                   'wavelength': 1,
                   'replicate': 1}
    name_1 = 'TEST1~wavelength-1~replicate-0'
    name_2 = 'TEST1~wavelength-1~replicate-1'
    data_dict = {
        name_1: fudge_data_1,
        name_2: fudge_data_2}
    experiment = Experiment(name='TEST1', kind='test',
                     frequency=frequency,
                     wavelengths=wavelengths,
                     temperatures=temperatures)
    yield {
        'exp': experiment,
        'frequency': frequency,
        'wavelength': wavelengths,
        'temperature': temperatures,
        'name_1': name_1,
        'name_2': name_2,
        'fudge_data_1': fudge_data_1,
        'fudge_data_2': fudge_data_2,
        'condition_1': condition_1,
        'condition_2': condition_2,
        'data_dict': data_dict,}
    rmtree(path_data['data_base_path'], ignore_errors=True)
    rmtree(path_data['figures_base_path'], ignore_errors=True)
    rmtree(path_data['designs_base_path'], ignore_errors=True)

def testGenerateGroupsNone(exp):
    desired_groups = exp['data_dict']
    actual_groups = exp['exp'].generate_groups(
        data_dict=exp['data_dict'], group_along=None)
    assertDataDictEqual(actual_groups, desired_groups)

def testGenerateGroups(exp):

    desired_groups = {'TEST1~wavelength-1': {
        'TEST1~wavelength-1~replicate-0': None,
        'TEST1~wavelength-1~replicate-1': None,
    }}

    actual_groups = exp['exp'].generate_groups(
        data_dict=exp['data_dict'], group_along='replicate')

    assertDataDictEqual(actual_groups, desired_groups)

def testGenerateGroupsWrongGroup(exp):
    """
    Confirms that we raise a valueError if we are not able to find the
    group
    """
    with pytest.raises(ValueError):
        exp['exp'].generate_groups(
                data_dict=exp['data_dict'], group_along='wrong_name')

def test_generate_groups_value(exp):
    desired_groups = {
        'TEST1~replicate-0': {
             'TEST1~wavelength-1~replicate-0': None},
        'TEST1~replicate-1': {
            'TEST1~wavelength-1~replicate-1': None}}
    actual_groups = exp['exp'].generate_groups(
            data_dict=exp['data_dict'], group_along='replicate', grouping_type='value')
    assertDataDictEqual(actual_groups, desired_groups)

def test_group_data_none(exp):
    """
    Assert that when group_along is none we just pass through the data
    """
    desired_data = exp['data_dict']
    actual_data = exp['exp'].group_data(
        data_dict=exp['data_dict'], group_along=None)
    assertDataDictEqual(actual_data, desired_data)

def test_group_scalar_data(exp):
    """
    Tests whether we can compress the derived quantity as a bunch of
    scalar-only dicts into a bunch of plottable and usaable pandas array.
    """
    scalar_data = {'TEST1~wavelength-1~temperature-25': 1,
                   'TEST1~wavelength-1~temperature-35': 2}

    desired_data = {'TEST1~wavelength-1': {
        'TEST1~wavelength-1~temperature-25': 1,
        'TEST1~wavelength-1~temperature-35': 2,}}

    actual_data = exp['exp'].group_data(
        data_dict=scalar_data, group_along='temperature')

    assertDataDictEqual(actual_data, desired_data)

def test_group_by_name(exp):

    actual_data = exp['exp'].group_data(
        data_dict=exp['data_dict'], group_along='replicate', grouping_type='name')
    desired_data = {
        'TEST1~wavelength-1': {
            'TEST1~wavelength-1~replicate-0': exp['fudge_data_1'],
            'TEST1~wavelength-1~replicate-1': exp['fudge_data_2']}}

    # Assert all values inside the pandas array are identical
    assertDataDictEqual(actual_data, desired_data)

def test_group_by_value(exp):
    actual_data = exp['exp'].group_data(
            data_dict=exp['data_dict'], group_along='wavelength', grouping_type='value')
    desired_data = {
        'TEST1~wavelength-1': {
            'TEST1~wavelength-1~replicate-0': exp['fudge_data_1'],
            'TEST1~wavelength-1~replicate-1': exp['fudge_data_2'],}}
    assertDataDictEqual(actual_data, desired_data)
